#!/usr/bin/perl

use strict;

my @sani = <STDIN>;

my %known = ();
my %unknown = ();

foreach (@sani) {
  if (m/(\S+) join (\S+)/) {
    my $joinname = $2;
    $known{$joinname} = 1;
  } elsif (m/(\S+) leave (\S+)/) {
    my $quitname = $2;
    if (!$known{$quitname}) {
      $known{$quitname} = 1;
      $unknown{$quitname} = 1;
    }
  } elsif (m/(\S+) namechange (\S+) (\S+)/) {
    my $joinname = $3;
    $known{$joinname} = 1;
    my $quitname = $2;
    if (!$known{$quitname}) {
      $known{$quitname} = 1;
      $unknown{$quitname} = 1;
    }
  } elsif (m/(\S+) message (\S+) (\S+)/) {
    my $talkname = $2;
    if (!$known{$talkname}) {
      $known{$talkname} = 1;
      $unknown{$talkname} = 1;
    }
  } else {
    print "Bad sani: $_\n";
  }
}

my @allnames = ();

my $i = 1;
my $k;
foreach $k (sort keys %known) {
  push(@allnames, "$k\n");
  $known{$k} = $i;
  $i = $i + 1;
}

my %names = %unknown;

my @gout = ();

foreach (@sani) {
  my $group = 0; my $talk = 0; my $time; my $text;
  my $joinname; my $talkname; my $quitname;
  if (m/(\S+) join (\S+)/) {
    $time = $1;
    $joinname = $2;
    if ($names{$joinname} == 1) {
      print "WARNING: $time - $joinname double joined\n";
    } else {
      $names{$joinname} = 1;
    }
    $group = 1;
  } elsif (m/(\S+) leave (\S+)/) {
    $time = $1;
    $quitname = $2;
    if ($names{$quitname} == 0) {
      print "WARNING: $time - $quitname double quit\n";
    } else {
      $names{$quitname} = 0;
    }
    $group = 1;
  } elsif (m/(\S+) namechange (\S+) (\S+)/) {
    $time = $1;
    $joinname = $3;
    $quitname = $2;
    if ($names{$joinname} == 1) {
      print "WARNING: $time - $joinname double joined\n";
    } else {
      $names{$joinname} = 1;
    }
    if ($names{$quitname == 0}) {
      print "WARNING: $time - $quitname double quit\n";
    } else {
      $names{$quitname} = 0;
    }
    $group = 1;
  } elsif (m/(\S+) message (\S+) (\S+)/) {
    $time = $1;
    $talkname = $2;
    $text = $3;
    if ($names{$talkname} == 0) {
      print "WARNING: $time - $talkname talking after quitting\n";
    }
    $talk = 1;
  }
  if ($group == 1 || $talk == 1) {
    my $bitstr = "";
    foreach $k (sort keys %known) {
      if ($names{$k}) {
        $bitstr = "$bitstr 1";
      } else {
        $bitstr = "$bitstr 0";
      }
    }
    if ($time =~ m/(\d\d):(\d\d):(\d\d)/) {
      $time = $1 * 3600 + $2 * 60 + $3;
    }
    if ($group == 1) {
      push(@gout, "$time $bitstr\n");
    }
  }
}

print @gout;
