#!/usr/bin/perl

use strict;

my $DISPLAYWARNINGS = 0;

my $timestampre = '<a .*name=\"(\d\d:\d\d:\d\d).*<\/a> <font class=\".*?\">.*';
my $eventre = '<a .*name=\"\d\d:\d\d:\d\d.*<\/a> <font class=\"(.*?)\">.*';
my $bodyre = '<a .*name=\"\d\d:\d\d:\d\d.*<\/a> <font class=\".*?\">(.*)';

my @eventstrings = ("mn","ml","mk","ts","mb","mj","mnc","mne","msc","mrcm");
my @eventtypes = ("message","leave","leave","leave","leave","join","namechange","none","none","none");
my @eventres = ('&lt;(.*)&gt;<\/font> (.*)<br\/>',
  '(.*) leaves the room.*<\/font><br\/>',
  '(.*) has been kicked.*<\/font><br\/>',
  '(.*) has been banned.*<\/font><br\/>',
  '(.*) has been banned.*<\/font><br\/>',
  '(.*) joins the room.*<\/font><br\/>',
  '(.*) is now known as (.*)<\/font><br\/>',
  '',
  '',
  '');

use Digest::HMAC_SHA1  qw(hmac_sha1 hmac_sha1_hex);

my $file; my $key = ""; my @names = ();

if (scalar(@ARGV) < 1) {
  die "No args given\nUsage: sanitize.pl <logfile> [secret key] [names file]\n";
} elsif (scalar(@ARGV) == 1) {
  $file = $ARGV[0];
} elsif (scalar(@ARGV) == 2) {
  $file = $ARGV[0];
  $key = $ARGV[1];
} elsif (scalar(@ARGV) == 3) {
  $file = $ARGV[0];
  $key = $ARGV[1];
  my $namesfile = $ARGV[2];
  open (NFILE, "<$namesfile") or die ("Cannot open file: $namesfile\n");
  @names = <NFILE>;
  close NFILE;
  if ($names[0] * $names[1] + 2 != scalar @names) {
    die ("Malformed names file\n");
  }
}

open (INFILE, "<$file") or die ("Cannot open file: $file\n");
my @log = <INFILE>;
close INFILE;

my @sani = ();

foreach (@log) {
  my $time; my $event; my $body;
  my $line = $_;
  if ($line =~ m/$timestampre/) {
    $time = $1;
    if ($line =~ m/$eventre/) {
      $event = $1;
      if ($line = m/$bodyre/) {
        $body = $1;
      } else {
        if ($DISPLAYWARNINGS) { print "No body found:\n$_\n"; }
      }
    } else {
      if ($DISPLAYWARNINGS) { print "No event found:\n$_\n"; }
    }
  } else {
    if ($DISPLAYWARNINGS) { print "No timestamp found:\n$_\n"; }
  }

  for(my $i=0; $i<scalar(@eventstrings); $i++) {
    if ($event eq $eventstrings[$i]) {
      if ($body =~ m/$eventres[$i]/) {
        if ($eventtypes[$i] eq "message") {
          my $name = anonize($1, $key, @names);      
          my $text = length($2); 
          push(@sani, "$time message $name $text\n");
        } elsif ($eventtypes[$i] eq "leave") {
          my $name = anonize($1, $key, @names);
          push(@sani, "$time leave $name\n");
        } elsif ($eventtypes[$i] eq "join") {
          my $name = anonize($1, $key, @names);
          push(@sani, "$time join $name\n");
        } elsif ($eventtypes[$i] eq "namechange") {
          my $oldname = anonize($1, $key, @names);
          my $newname = anonize($2, $key, @names);
          push(@sani, "$time namechange $oldname $newname\n");
        }
      } else {
        if ($DISPLAYWARNINGS) { print "Bad parse:\n$_\n"; }
      }
    }
  }
}

open (SANI, ">$file.sani") or die ("Cannot write to file: $file.sani\n");
print SANI @sani;
close(SANI);


sub anonize (\$$@) {
  (my $name, my $key, my @names) = @_;
  chomp(@names);

  $name =~ s/\"/\'/g;

  if ($key eq "") {
    return "\"$name\"";
  } elsif (scalar @names == 0) {
    return hmac_sha1_hex($name, $key);
  } else {
    my $anon = "";
    my $incrmax = $names[0];
    my $incr = 0;
    my $mod = $names[1];
    my $numchars = int(log($mod)/log(16)+1);
    my $hash = hmac_sha1_hex($name, $key);

    while ($incr < $incrmax) {
      my $subhash = substr $hash, -$numchars*($incr+1), $numchars; 
      $anon = $anon . $names[hex($subhash) % $mod + $incr * $mod + 2];
      $incr = $incr + 1;
    }

    return "\"$anon\"";
  }
}
